<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 15/04/19
 * Time: 16.06
 */

namespace FiloBlu\Hotfix\Plugin\Model\Rule\Condition;

use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Model\AbstractModel;

/**
 * Class Product
 * @package FiloBlu\Hotfix\Plugin\Model\Rule\Condition
 */
class Product
{
    /**
     * @var string
     */
    protected $couponCode;
    /**
     * @var ConfigurableModel
     */
    protected $configurableModel;
    /**
     * @var ProductRepository
     */
    protected $productRepository;
    /**
     * @var ProductMetadataInterface
     */
    protected $productMetadata;

    /**
     * Product constructor.
     * @param ConfigurableModel $configurableModel
     * @param ProductRepository $productRepository
     * @param ProductMetadataInterface $productMetadata
     */
    public function __construct(Configurable $configurableModel,
                                ProductRepositoryInterface $productRepository,
                                ProductMetadataInterface $productMetadata)
    {
        $this->configurableModel = $configurableModel;
        $this->productRepository = $productRepository;
        $this->productMetadata = $productMetadata;

    }

    /**
     * @param \Magento\SalesRule\Model\Rule\Condition\Product $product
     * @param callable $proceed
     * @param AbstractModel $model
     * @return bool
     */
    public function aroundValidate(\Magento\SalesRule\Model\Rule\Condition\Product $product, callable $proceed, AbstractModel $model)
    {

        //as written in \Magento\SalesRule\Model\Rule\Condition\Product :: validate
        ////@todo reimplement this method when is fixed MAGETWO-5713
        //MAGETWO-5713 merged into 2.3.1 . For more info see https://github.com/magento/magento2/commit/3de5d0e6e6df9e5d860fbdd1750347d7d606a204
        $myResult = false;

        if ($this->checkIfToApplyPlugin() && $product->getAttribute() === 'category_ids')
        {

            $operator = $product->getOperator();
            $category = $product->getValueParsed();

            $this->couponCode = $model->getQuote()->getCouponCode();
            $myProduct = $model->getProduct();
            if($myProduct->getTypeId() == 'simple' && $myProduct->getVisibility() == \Magento\Catalog\Model\Product\Visibility::VISIBILITY_NOT_VISIBLE) {
                $parentConfigObject = $this->configurableModel->getParentIdsByChild($myProduct->getId());
                if($parentConfigObject) {
                    $configurableId = $parentConfigObject[0];
                    $configurableProduct = $this->productRepository->getById($configurableId);
                    $myProductCategory = $configurableProduct->getCategoryIds();
                }
            }
            else {
                $myProductCategory = $myProduct->getCategoryIds();
            }

            //change operator for is one of and is not of one
            $operator = str_replace('()', '{}', $operator);

            switch ($operator) {
                case '{}' :
                    foreach ($category as $value) {
                        if (in_array($value, $myProductCategory, false)) {
                            $myResult = true;
                        }
                    }
                    break;

                case '!{}' :
                    foreach ($category as $value) {
                        if (!in_array($value, $myProductCategory, false)) {
                            $myResult = true;
                        }
                    }
                    break;

                case '==' :
                    if ($category === $myProductCategory) {
                        $myResult = true;
                    }
                    break;

                case '!=' :
                    if ($category !== $myProductCategory) {
                        $myResult = true;
                    }
                    break;
            }

            return $myResult;
        }
        else
        {
            return $proceed($model);
        }
    }

    /**
     * @return bool
     */
    public function checkIfToApplyPlugin()
    {
        if ($this->getVersion() < '2.3.1') {
            return true;
        }

        return false;
    }

    /**
     * @return string
     */
    public function getVersion()
    {
        return $this->productMetadata->getVersion();
    }

}